#--
################################################################################
#                      Copyright (C) 2011 Travis Herrick                       #
################################################################################
#                                                                              #
#                                 \v^V,^!v\^/                                  #
#                                 ~%       %~                                  #
#                                 {  _   _  }                                  #
#                                 (  *   -  )                                  #
#                                 |    /    |                                  #
#                                  \   _,  /                                   #
#                                   \__.__/                                    #
#                                                                              #
################################################################################
# This program is free software: you can redistribute it                       #
# and/or modify it under the terms of the GNU General Public License           #
# as published by the Free Software Foundation,                                #
# either version 3 of the License, or (at your option) any later version.      #
#                                                                              #
# Commercial licensing may be available for a fee under a different license.   #
################################################################################
# This program is distributed in the hope that it will be useful,              #
# but WITHOUT ANY WARRANTY;                                                    #
# without even the implied warranty of MERCHANTABILITY                         #
# or FITNESS FOR A PARTICULAR PURPOSE.                                         #
# See the GNU General Public License for more details.                         #
#                                                                              #
# You should have received a copy of the GNU General Public License            #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.        #
################################################################################
#++

require_relative 'require'

class ExitTest < Test::Unit::TestCase

  def setup
    override_exit(true)
    reset_app_state
  end

  def teardown
    override_exit(false)
  end

  def test_without_override
    override_exit(false)
    assert_raise(SystemExit) { exit }
    assert_dead
  end

  def test_basic_exit_call
    assert_alive
    assert_nothing_raised { exit }
    assert_dead
  end

  def test_exit_parameter
    assert_alive
    assert_nothing_raised { exit(-1) }
    assert_dead
  end

  def test_exit_return_value
    out = nil
    assert_alive
    assert_nothing_raised { out = exit }
    assert_dead
    assert_equal true, out

    assert_alive
    assert_nothing_raised { out = exit(-1) }
    assert_dead
    assert_equal(-1, out)

    assert_alive
    assert_nothing_raised { out = exit('blah') }
    assert_dead
    assert_equal 'blah', out
  end

  ############################################################################
  private
  ############################################################################

  def override_exit(value)
    if value
      OverrideKernel.start_override_exit
    else
      OverrideKernel.stop_override_exit
    end
  end

  def reset_app_state
    OverrideKernel::AppState.state = :alive
  end

  def assert_alive
    alive = OverrideKernel::AppState.alive
    reset_app_state unless alive
    assert alive, 'The application is not running when it was expected to be.'
  end

  def assert_dead
    dead = OverrideKernel::AppState.dead
    reset_app_state if dead
    assert dead, 'The application is running when it was not expected to be.'
  end
end
