#--
################################################################################
#                      Copyright (C) 2011 Travis Herrick                       #
################################################################################
#                                                                              #
#                                 \v^V,^!v\^/                                  #
#                                 ~%       %~                                  #
#                                 {  _   _  }                                  #
#                                 (  *   -  )                                  #
#                                 |    /    |                                  #
#                                  \   _,  /                                   #
#                                   \__.__/                                    #
#                                                                              #
################################################################################
# This program is free software: you can redistribute it                       #
# and/or modify it under the terms of the GNU General Public License           #
# as published by the Free Software Foundation,                                #
# either version 3 of the License, or (at your option) any later version.      #
#                                                                              #
# Commercial licensing may be available for a fee under a different license.   #
################################################################################
# This program is distributed in the hope that it will be useful,              #
# but WITHOUT ANY WARRANTY;                                                    #
# without even the implied warranty of MERCHANTABILITY                         #
# or FITNESS FOR A PARTICULAR PURPOSE.                                         #
# See the GNU General Public License for more details.                         #
#                                                                              #
# You should have received a copy of the GNU General Public License            #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.        #
################################################################################
#++

require_relative 'require'

class OverrideKernelTest < Test::Unit::TestCase
  ITEMS = {
    :method_list => [:override_exit,],
    :constants => {
      :AppState => StateManager.new(:alive, [:alive, :dead]),
    },
  }

  def setup
    @module = OverrideKernel
    @module.instance_eval do
      instance_variables.each do |var|
        remove_instance_variable var
      end
    end
    super
  end

  def test_constants
    ITEMS[:constants].each_key do |constant|
      assert_equal strip_id(ITEMS[:constants][constant]),
        strip_id(@module.const_get(constant))
    end
  end

  def test_method_list
    assert_equal ITEMS[:method_list], @module.send(:method_list)
  end

  def test_all_methods
    ITEMS[:method_list].each do |method|
      assert_equal nil, @module.send(method)

      {
        "start_#{method}".to_sym => true,
        "stop_#{method}".to_sym => false,
      }.each do |method_name, value|
        assert_respond_to @module, method_name
        assert_nothing_raised { @module.send method_name }
        assert_respond_to @module, method
        assert_equal value, @module.send(method)
      end
    end

    assert_no_response @module, :method_list, false
  end

  def test_module_include
    ITEMS[:method_list].each do |method|
      assert_no_response TestModule, method

      [
        "start_#{method}".to_sym,
        "stop_#{method}".to_sym,
      ].each do |meth|
        assert_no_response TestModule, meth
      end
    end

    ITEMS[:constants].each_key do |constant|
      assert_equal strip_id(ITEMS[:constants][constant]),
        strip_id(TestModule.const_get(constant))
    end

    assert_no_response TestModule, :method_list
  end

  def test_class_include
    obj = TestClass.new
    ITEMS[:method_list].each do |method|
      assert_no_response TestClass, method
      assert_no_response obj, method

      [
        "start_#{method}".to_sym,
        "stop_#{method}".to_sym,
      ].each do |meth|
        assert_no_response TestClass, meth
        assert_no_response obj, meth
      end
    end

    ITEMS[:constants].each_key do |constant|
      assert_equal strip_id(ITEMS[:constants][constant]),
        strip_id(TestClass.const_get(constant))
    end

    assert_no_response TestClass, :method_list
    assert_no_response obj, :method_list
  end

  ############################################################################
  private
  ############################################################################

  def strip_id(object)
    return object unless object.respond_to?(:pretty_inspect) &&
      object.pretty_inspect.match(/^#</)
    object.pretty_inspect.sub(/:\w+? /, ' ')
  end

  def assert_no_response(obj, method, include_private = true, msg = nil)
    msg = message(msg) {
      "Did not expect #{mu_pp(obj)} (#{obj.class}) to respond to ##{method}"
    }
    assert !obj.respond_to?(method, include_private), msg
  end
end
